require 'time'
require File.join(File.expand_path(File.dirname(__FILE__)), "..", "..", "..", "spec_helper")

describe BillForward::Amendment do
	before :all do
		@client = BillForwardTest::TEST_CLIENT
		BillForward::Client.default_client = @client
		@invoice_id = 'mocking this anyway'
		@subscription_id = 'mocking this anyway'
	end
	describe '::create' do
		context 'with malformed request body' do
			it "produces a 400 Server Error" do
				# we will mock the serialization anyway
				subscription = BillForward::Subscription.new

				time_string = Time.now.iso8601.sub! '+00:00', 'Z'

			    amendment = BillForward::InvoiceRecalculationAmendment.new({
					'subscription' => subscription,
					'invoiceID' => @invoice_id,
					"invoiceVersionID" => @invoice_id,
					'newInvoiceState' => 'Pending',
					"actioningTime" => time_string
				})

				amendment = double
				allow(amendment).to receive(:_client).and_return(@client)
				allow(amendment.class).to receive(:resource_path).and_return(BillForward::InvoiceRecalculationAmendment.resource_path)
			    allow(amendment).to receive(:serialize).and_return(malformed_serialize)

				expect{BillForward::InvoiceRecalculationAmendment.create(amendment)}.to raise_error(BillForward::ApiError, /400|500/)
			end
		end
	end
end

def malformed_serialize
	# produces a request body where some nested entities have @type key in a non-top position within their JSON.
	# specifically the @type on flatPricingComponent is not, in this request, the first key within its object.
	# this is known to cause Server Error. (API prints Exception Description: Missing class indicator field from database row [UnmarshalRecordImpl()])
'{"@type":"InvoiceRecalculationAmendment","newInvoiceState":"Pending","invoiceID":"B0ED27EF-B956-4AE8-9FD2-9A5BDFA551B3","subscription":{"@type":"subscription","name":"Memorable Subscription","description":"Memorable Subscription Description","currentPeriodEnd":"2014-09-24T17:22:02Z","initialPeriodStart":"2014-09-24T17:21:02Z","fixedTerms":[],"changedBy":"System","id":"ACD66517-6F32-44CB-AF8C-3097F97E1E67","managedBy":"BillForward","organizationID":"7F3D3A3C-BAA4-4698-9645-EC33F853B3D8","subscriptionEnd":"2014-09-24T17:22:02Z","accountID":"14384081-3A24-460A-AE67-40E0488B267A","pricingComponentValues":[{"changedBy":"614E626E-72BF-4B69-B40E-0B72A1BB7CF4","id":"29F353B6-BE3E-4E31-8E26-84F70F3FB0F3","organizationID":"7F3D3A3C-BAA4-4698-9645-EC33F853B3D8","pricingComponentID":"71753C8E-E477-4DF6-8C32-20FACA3E5B20","updated":"2014-09-24T17:21:02Z","subscriptionID":"ACD66517-6F32-44CB-AF8C-3097F97E1E67","created":"2014-09-24T17:21:02Z","value":1},{"changedBy":"614E626E-72BF-4B69-B40E-0B72A1BB7CF4","id":"8597315B-A9F7-45C4-B145-DF6584528B02","organizationID":"7F3D3A3C-BAA4-4698-9645-EC33F853B3D8","pricingComponentID":"C37D16B1-5E39-4B75-BEC0-03AB4085941E","updated":"2014-09-24T17:21:02Z","subscriptionID":"ACD66517-6F32-44CB-AF8C-3097F97E1E67","created":"2014-09-24T17:21:02Z","value":5}],"type":"Subscription","currentPeriodStart":"2014-09-24T17:21:02Z","pricingComponentValueChanges":[],"updated":"2014-09-24T17:22:04Z","successfulPeriods":1,"productRatePlan":{"name":"A sound plan","validFrom":"2014-09-24T17:21:00Z","changedBy":"614E626E-72BF-4B69-B40E-0B72A1BB7CF4","id":"BFE95484-D1D0-4296-ABB8-C2A3D4CE95EF","organizationID":"7F3D3A3C-BAA4-4698-9645-EC33F853B3D8","taxation":[],"product":{"name":"Month of Paracetamoxyfrusebendroneomycin","description":"It can cure the common cold and being struck by lightning","durationPeriod":"minutes","deleted":false,"changedBy":"614E626E-72BF-4B69-B40E-0B72A1BB7CF4","id":"0CE0A471-A8B1-4E33-B5F4-115947DE8C55","organizationID":"7F3D3A3C-BAA4-4698-9645-EC33F853B3D8","productType":"non-recurring","duration":1,"trial":0,"updated":"2014-09-24T17:20:59Z","created":"2014-09-24T17:20:59Z","trialPeriod":"none","state":"prod"},"proRataMode":"WithCoupon","pricingComponents":[{"name":"Devices used, fixed","description":"How many devices you use, I guess","chargeType":"subscription","defaultQuantity":1,"minQuantity":0,"validFrom":"2014-09-24T17:21:00Z","changedBy":"614E626E-72BF-4B69-B40E-0B72A1BB7CF4","id":"71753C8E-E477-4DF6-8C32-20FACA3E5B20","chargeModel":"flat","downgradeMode":"immediate","organizationID":"7F3D3A3C-BAA4-4698-9645-EC33F853B3D8","versionID":"71753C8E-E477-4DF6-8C32-20FACA3E5B20","updated":"2014-09-24T17:21:01Z","unitOfMeasureID":"6EB5B013-AFD3-4FC6-BDED-73FC681F23EE","upgradeMode":"immediate","created":"2014-09-24T17:21:01Z","tiers":[{"changedBy":"614E626E-72BF-4B69-B40E-0B72A1BB7CF4","id":"EA6A046F-B436-4D5F-A4CE-14638E2A574C","organizationID":"7F3D3A3C-BAA4-4698-9645-EC33F853B3D8","price":1.0,"pricingComponentID":"71753C8E-E477-4DF6-8C32-20FACA3E5B20","lowerThreshold":1,"pricingType":"fixed","created":"2014-09-24T17:21:01Z","pricingComponentVersionID":"71753C8E-E477-4DF6-8C32-20FACA3E5B20","productRatePlanID":"BFE95484-D1D0-4296-ABB8-C2A3D4CE95EF","upperThreshold":1}],"unitOfMeasure":{"name":"Devices","deleted":false,"changedBy":"614E626E-72BF-4B69-B40E-0B72A1BB7CF4","id":"6EB5B013-AFD3-4FC6-BDED-73FC681F23EE","organizationID":"7F3D3A3C-BAA4-4698-9645-EC33F853B3D8","roundingScheme":"UP","updated":"2014-09-24T17:20:58Z","created":"2014-09-24T17:20:58Z","displayedAs":"Devices"},"productRatePlanID":"BFE95484-D1D0-4296-ABB8-C2A3D4CE95EF","@type":"flatPricingComponent"},{"name":"Devices used, tiered","description":"How many devices you use, but with a tiering system","chargeType":"usage","defaultQuantity":10,"minQuantity":0,"validFrom":"2014-09-24T17:21:00Z","changedBy":"614E626E-72BF-4B69-B40E-0B72A1BB7CF4","id":"C37D16B1-5E39-4B75-BEC0-03AB4085941E","chargeModel":"tiered","downgradeMode":"immediate","organizationID":"7F3D3A3C-BAA4-4698-9645-EC33F853B3D8","versionID":"C37D16B1-5E39-4B75-BEC0-03AB4085941E","updated":"2014-09-24T17:21:01Z","unitOfMeasureID":"6EB5B013-AFD3-4FC6-BDED-73FC681F23EE","upgradeMode":"immediate","created":"2014-09-24T17:21:01Z","tiers":[{"changedBy":"614E626E-72BF-4B69-B40E-0B72A1BB7CF4","id":"7B86D8A6-CFB1-4CD9-B438-617F4E060C3A","organizationID":"7F3D3A3C-BAA4-4698-9645-EC33F853B3D8","price":5.0,"pricingComponentID":"C37D16B1-5E39-4B75-BEC0-03AB4085941E","lowerThreshold":2,"pricingType":"unit","created":"2014-09-24T17:21:01Z","pricingComponentVersionID":"C37D16B1-5E39-4B75-BEC0-03AB4085941E","productRatePlanID":"BFE95484-D1D0-4296-ABB8-C2A3D4CE95EF","upperThreshold":10},{"changedBy":"614E626E-72BF-4B69-B40E-0B72A1BB7CF4","id":"B3E8C950-A71E-434D-8988-1F964430768B","organizationID":"7F3D3A3C-BAA4-4698-9645-EC33F853B3D8","price":10.0,"pricingComponentID":"C37D16B1-5E39-4B75-BEC0-03AB4085941E","lowerThreshold":1,"pricingType":"fixed","created":"2014-09-24T17:21:01Z","pricingComponentVersionID":"C37D16B1-5E39-4B75-BEC0-03AB4085941E","productRatePlanID":"BFE95484-D1D0-4296-ABB8-C2A3D4CE95EF","upperThreshold":1},{"changedBy":"614E626E-72BF-4B69-B40E-0B72A1BB7CF4","id":"D05839BB-8E35-48FD-AB66-F1CDF0E59D9C","organizationID":"7F3D3A3C-BAA4-4698-9645-EC33F853B3D8","price":2.0,"pricingComponentID":"C37D16B1-5E39-4B75-BEC0-03AB4085941E","lowerThreshold":11,"pricingType":"unit","created":"2014-09-24T17:21:01Z","pricingComponentVersionID":"C37D16B1-5E39-4B75-BEC0-03AB4085941E","productRatePlanID":"BFE95484-D1D0-4296-ABB8-C2A3D4CE95EF","upperThreshold":100}],"unitOfMeasure":{"name":"Devices","deleted":false,"changedBy":"614E626E-72BF-4B69-B40E-0B72A1BB7CF4","id":"6EB5B013-AFD3-4FC6-BDED-73FC681F23EE","organizationID":"7F3D3A3C-BAA4-4698-9645-EC33F853B3D8","roundingScheme":"UP","updated":"2014-09-24T17:20:58Z","created":"2014-09-24T17:20:58Z","displayedAs":"Devices"},"productRatePlanID":"BFE95484-D1D0-4296-ABB8-C2A3D4CE95EF","@type":"tieredPricingComponent"}],"updated":"2014-09-24T17:21:00Z","currency":"USD","taxStatus":"inclusive","created":"2014-09-24T17:21:00Z","productID":"0CE0A471-A8B1-4E33-B5F4-115947DE8C55","fixedTermDefinitions":[]},"created":"2014-09-24T17:21:02Z","paymentMethodSubscriptionLinks":[{"deleted":false,"changedBy":"614E626E-72BF-4B69-B40E-0B72A1BB7CF4","id":"1A89116E-D17D-4610-B139-6E4559CC85FD","organizationID":"7F3D3A3C-BAA4-4698-9645-EC33F853B3D8","updated":"2014-09-24T17:21:02Z","subscriptionID":"ACD66517-6F32-44CB-AF8C-3097F97E1E67","created":"2014-09-24T17:21:02Z","paymentMethodID":"612D5A60-4F6A-455D-9C03-685A6D96A8D4","paymentMethod":{"name":"Credit Notes","description":"Pay using credit","userEditable":false,"deleted":false,"changedBy":"614E626E-72BF-4B69-B40E-0B72A1BB7CF4","id":"612D5A60-4F6A-455D-9C03-685A6D96A8D4","priority":100,"organizationID":"7F3D3A3C-BAA4-4698-9645-EC33F853B3D8","reusable":true,"accountID":"14384081-3A24-460A-AE67-40E0488B267A","linkID":"","updated":"2014-09-24T17:20:56Z","created":"2014-09-24T17:20:56Z","gateway":"credit_note"}}],"state":"Expired","productRatePlanID":"BFE95484-D1D0-4296-ABB8-C2A3D4CE95EF","productID":"0CE0A471-A8B1-4E33-B5F4-115947DE8C55"},"invoiceVersionID":"B0ED27EF-B956-4AE8-9FD2-9A5BDFA551B3","actioningTime":"2014-09-25T18:34:46Z"}'
end